#% constants
module SiSUversion
  SiSU_version_next_stable           = '5.7.1'  #% set version stable current
  SiSU_version_next_unstable         = '6.3.1'  #% set version unstable dev
  #% qi quick install file
  SiSU_version_generic_next_stable   = '5.4.x'
  SiSU_version_generic_next_unstable = '6.0.x'
  SiSU_version_dir_stable            = 'v5'
  SiSU_version_dir_unstable          = 'v6'
end
module Dev
  GPGpubKey                          = '1BB4B289'
end
