module Help
  def self.project_help
    puts <<WOK

#{Project_details.name}
  #{Project_details.rake_rant}
  #{Project_details.platform_notice}

Commands quick start list
  #{Project_details.name} Rake/Rant Help: (Rakefile or Rantfile)
    rake (rake -T or rant -T)                  # a task list, (generated by Rake or Rant) for more complete and up to date help

  Quick start install and remove project #{Project_details.name}, as root:
    rake setup                                  # install #{Project_details.name}
    rake resetup                                # reinstall #{Project_details.name}
    rake remove_package                         # clobber/remove #{Project_details.name}, all versions
  alternatively to install as a gem:
    rake gem_create_build_install               # create gemspec; build gem, &; install gem, for #{Project_details.name}

  NOTE: these install options DO NOT setup #{Project_details.name} dependencies for:
  LaTeX(XeTeX) pdf output; or
  databases (postgresql or sqlite3) for #{Project_details.name} search
  should you wish to produce outputs that depend on them
  (on Debian for example this is the work of the debian installer)

WOK
  end
  def self.tasks(make='rake')
    begin
      system("#{make} -T")
    rescue
      puts 'is either rake or rant installed?'
    end
  end
end

