:set number
:set relativenumber
:autocmd FileType    ruby :set relativenumber
:autocmd FocusLost   *.rb :set norelativenumber
:autocmd FocusGained *.rb :set relativenumber
:autocmd InsertEnter *.rb :set norelativenumber
:autocmd InsertLeave *.rb :set relativenumber
