" Vim color file
" Name:            frugal-cterm
" Maintainer:      Ralph Amissah <ralph@amissah.com>
" Last Change:     2013-02-09
" URL:             <http://git.sisudoc.org/gitweb/?p=code/sisu.git;a=blob;f=data/sisu/conf/editor-syntax-etc/vim/colors/frugal-cterm.vim;hb=HEAD>
" Note:            8 color cterm, related colorschemes 8 & sparse
" http://vim.wikia.com/wiki/Xterm256_color_names_for_console_Vim
" http://guns.github.com/xterm-color-table.vim/images/xterm-color-table-with-visible-rgb.png
:set t_Co=256
:set background=dark
:hi clear
if exists("syntax_on")
  syntax reset
endif
:let colors_name = "frugal-cterm"
" 0 = black, 1 = red, 2 = green, 3 = yellow/orange, 4 = blue, 5 = magenta, 6 = cyan, 7 = white
:hi Normal                                                           ctermbg=0           ctermfg=7
:hi Cursor         cterm=reverse
:hi lCursor        cterm=reverse
:hi StatusLine     cterm=bold,reverse
:hi StatusLineNC   cterm=reverse
:hi Search         cterm=none          ctermbg=2           ctermfg=0
:hi IncSearch      cterm=bold          ctermbg=2           ctermfg=0
:hi SpecialKey                                             ctermfg=4
:hi Visual         cterm=reverse
:hi VisualNOS      cterm=bold,underline
:hi MoreMsg                                                ctermfg=2
:hi ModeMsg        cterm=bold
:hi Question                                               ctermfg=2
:hi Title          cterm=bold                              ctermfg=3
:hi NonText        cterm=bold                              ctermfg=4
:hi LineNr         cterm=bold          ctermbg=0           ctermfg=0*
:hi Directory                                              ctermfg=4
:hi WildMenu                           ctermbg=3           ctermfg=0
:hi VertSplit      cterm=reverse
:hi Folded         cterm=none          ctermbg=0           ctermfg=7
:hi FoldColumn                         ctermbg=7           ctermfg=4
:hi DiffChange     cterm=none          ctermbg=7           ctermfg=0
:hi DiffText       cterm=none          ctermbg=6           ctermfg=0
:hi DiffAdd        cterm=none          ctermbg=2           ctermfg=0
:hi DiffDelete     cterm=none          ctermbg=7           ctermfg=0
:hi String         cterm=none                              ctermfg=3
:hi Comment        cterm=none          ctermbg=0           ctermfg=7
:hi Constant                                               ctermfg=1
:hi Special                                                ctermfg=6
:hi Identifier                                             ctermfg=6
:hi Statement                                              ctermfg=2
:hi Operator                                               ctermfg=2
:hi PreProc                                                ctermfg=1
:hi Type           cterm=bold                              ctermfg=3
:hi Delimiter      cterm=none                              ctermfg=2
:hi Ignore         cterm=bold                              ctermfg=7
:hi Todo                               ctermbg=3           ctermfg=0
:hi Underlined     cterm=underline
:hi Include                                                ctermfg=1
:hi Define                                                 ctermfg=3
:hi Function                                               ctermfg=6
:hi Structure                                              ctermfg=2
:hi MatchParen     cterm=bold          ctermbg=4           ctermfg=7
:hi CursorLine     cterm=bold,underline
:hi CursorColumn   cterm=bold
:hi ColorColumn                        ctermbg=8
:hi SpellBad       cterm=none          ctermbg=7           ctermfg=0
:hi SpellCap       cterm=none          ctermbg=7           ctermfg=0
:hi SpellLocal     cterm=none          ctermbg=7           ctermfg=0
:hi SpellRare      cterm=none          ctermbg=7           ctermfg=0
:hi WarningMsg                                             ctermfg=1
:hi ErrorMsg       cterm=bold          ctermbg=1           ctermfg=7
:hi Error          cterm=bold          ctermbg=1           ctermfg=7
